<?php

/**
 * Keywords
 * here you can define some keywords and
 * stopwords
 *
 * Keywords explanation:
 * Keywords are words that can be used
 * when writing a query in natural language
 * lets take for example the units array
 * The keys are the words that the user can write
 * and they will be converted to the correct value
 * if the user types:
 * 100 centimeters to meters
 * the code will transform that query to
 * 100 cm to m
 *
 * Stop words explanation:
 * stop words are words that can be used in the query
 * when using natural languge for example
 * 100km to cm - here the stop word is "to"
 *
 * Stop words are used to check if the query
 * has to be processed for example
 * 100km equals meters
 * if the word "equals" is not registered in the stop_words array
 * then it won't be processd.
 * At the end this stop words are removed so
 * 100km to cm becomes 100km cm
 */

return [
    'units' => [
        'timmar' => 'hr',
        'timme' => 'hr',
        'sekunder' => 's',
        'sekund' => 's',
        'sek' => 's',
        'minuter' => 'min',
        'minut' => 'min',
        'år' => 'year',
        'månader' => 'month',
        'veckor' => 'week',
        'dagar' => 'day',
        'kelvin' => 'k',
        'fahrenheit' => 'f',
        'celsius' => 'c',
        'meter' => 'm',
        'centimeter' => 'cm',
        'kilometer' => 'km',
        'millimeter' => 'mm',
        'mikrometers' => 'μm',
        'miles per timme' => 'mph',
        'knop' => 'knot',
        'miles' => 'mi',
        'mile' => 'mi',
        'nautisk mil' => 'nmi',
        'nm' => 'nmi',
        'distans' => 'nmi',
        'distansminut' => 'nmi',
        'yard' => 'yd',
        '\'' => 'ft',
        'fot' => 'ft',
        '\"' => 'in',
        'tum' => 'in',
        'liter' => 'l',
        'pint' => 'pt',
        'us pints' => 'uspt',
        'us-pints' => 'uspt',
        'us pint' => 'uspt',
        'us-pint' => 'uspt',
        'uspints' => 'uspt',
        'uspint' => 'uspt',
        'uk pints' => 'pt',
        'uk-pints' => 'pt',
        'uk pint' => 'ukpt',
        'uk-pint' => 'ukpt',
        'ukpints' => 'ukpt',
        'ukpint' => 'ukpt',
        'uk pt' => 'ukpt',
        'ukpt' => 'ukpt',
        'uk quart' => 'ukqt',
        'uk-quart' => 'ukqt',
        'uk qt' => 'ukqt',
        'uk-qt' => 'ukqt',
        'ukquart' => 'ukqt',
        'us quart' => 'usqt',
        'us-quart' => 'usqt',
        'us qt' => 'usqt',
        'us-qt' => 'usqt',
        'usquart' => 'usqt',
        'gallons' => 'gal',
        'gallon' => 'gal',
        'gram' => 'g',
        'kilogram' => 'kg',
        'milligram' => 'mg',
        'ounces' => 'oz',
        'ounce' => 'oz',
        'pounds' => 'lb',
        'pound' => 'lb',
        'pund' => 'lb',
        'lbs' => 'lb',
        'fluid ounces' => 'floz',
        'fluid ounce' => 'floz',
        'flytande pund' => 'floz',
        'us floz' => 'floz',
        'us-floz' => 'floz',
        'us gallon' => 'usgal',
        'us-gallon' => 'usgal',
        'us gal' => 'usgal',
        'us-gal' => 'usgal',
        'usgallon' => 'usgal',
        'usgal' => 'usgal',
        'uk gallon' => 'ukgal',
        'uk-gallon' => 'ukgal',
        'uk gal' => 'ukgal',
        'uk-gal' => 'ukgal',
        'ukgallon' => 'ukgal',
        'ukgal' => 'ukgal',
        'watt' => 'w',
        'kilowatt' => 'kw',
        'hästkraft' => 'ps',
        'hästkrafter' => 'ps',

        'stop_words' => ['till', '=', 'i', 'som']
    ],
    'datastorage' => [
        'bytes' => 'b',
        'byte' => 'b',
        'kilobytes' => 'kb',
        'kilobyte' => 'kb',
        'megabytes' => 'mb',
        'megabyte' => 'mb',
        'megab' => 'mb',
        'gigabytes' => 'gb',
        'gigabyte' => 'gb',
        'gigab' => 'gb',
        'terabytes' => 'tb',
        'terabyte' => 'tb',
        'terab' => 'tb',
        'petabytes' => 'pb',
        'petabyte' => 'pb',
        'petab' => 'pb',
        'exabytes' => 'eb',
        'exabyte' => 'eb',
        'exab' => 'eb',
        'zettabytes' => 'zb',
        'zettabyte' => 'zb',
        'zettab' => 'zb',
        'yottabytes' => 'yb',
        'yottabyte' => 'yb',
        'yottab' => 'yb',
        'kibibytes' => 'kib',
        'kibibyte' => 'kib',
        'mebibytes' => 'mib',
        'mebibyte' => 'mib',
        'mebis' => 'mib',
        'mebi' => 'mib',
        'gibibytes' => 'gib',
        'gibibyte' => 'gib',
        'gibis' => 'gib',
        'gibi' => 'gib',
        'tebibytes' => 'tib',
        'tebibyte' => 'tib',
        'tebis' => 'tib',
        'tebi' => 'tib',
        'pebibytes' => 'pib',
        'pebibyte' => 'pib',
        'pebis' => 'pib',
        'pebi' => 'pib',
        'exbibytes' => 'eib',
        'exbibyte' => 'eib',
        'exbis' => 'eib',
        'exbi' => 'eib',
        'zebibytes' => 'zib',
        'zebibyte' => 'zib',
        'zebis' => 'zib',
        'zebi' => 'zib',
        'yobibytes' => 'yib',
        'yobibyte' => 'yib',
        'yobis' => 'yib',
        'yobi' => 'yib',

        'stop_words' => ['till', '=', 'i', 'som']
    ],
    'time' => [
        'plus' => '+',
        'minus' => '-',
        'imorgon' => '+1 day',
        'igår' => '-1 day',
        'arbetsdagar' => 'weekdays',
        'arbetsdag' => 'weekdays',
        'veckodagar' => 'weekdays',
        'veckodag' => 'weekdays',
        'tills' => ['until'],
        'mellan' => ['between'],
        'början av' => ['start of'],
        'starten av' => ['start of'],
        'slutet av' => ['end of'],
        'sluten av' => ['end of'],

        'stop_words' => ['till', '=', 'i', 'som']
    ],
    'currency' => [
        '€' => 'EUR',
        'euro' => 'EUR',
        'euros' => 'EUR',
        '¥' => 'JPY',
        'yen' => 'JPY',
        '$' => 'USD',
        'R$' => 'BRL',
        'лв' => 'BGN',
        '៛' => 'KHR',
        'C¥' => 'CNY',
        '₡' => 'CRC',
        '₱' => 'CUP',
        'Kč' => 'CZK',
        'kr' => 'SEK',
        'kronor' => 'SEK',
        'krona' => 'SEK',
        'RD$' => 'DOP',
        '£' => 'GBP',
        '¢' => 'GHS',
        'Q' => 'GTQ',
        'L' => 'HNL',
        'Ft' => 'HUF',
        'Rp' => 'IDR',
        '₹' => 'INR',
        'rupis' => 'INR',
        'rupies' => 'INR',
        'rupe' => 'INR',
        'rupee' => 'INR',
        'rupees' => 'INR',
        'rupiah' => 'IDR',
        'us dollars' => 'USD',
        'us dollar' => 'USD',
        'amerikanska dollar' => 'USD',
        'amerikansk dollar' => 'USD',
        'mexikanska pesos' => 'MXN',
        'mexikansk peso' => 'MXN',
        'argentinska pesos' => 'ARS',
        'argentinsk peso' => 'ARS',
        'colombianska pesos' => 'COP',
        'colombiansk peso' => 'COP',

        'kanadensiska dollar' => 'CAD',
        'kanadensisk dollar' => 'CAD',
        'canadian' => 'CAD',
        '₪' => 'ILS',
        'J$' => 'JMD',
        '₩' => 'KRW',
        'ден' => 'MKD',
        'RM' => 'MYR',
        'MT' => 'MZN',
        'ƒ' => 'ANG',
        'C$' => 'NIO',
        '₦' => 'NGN',
        'B/.' => 'PAB',
        'Gs' => 'PYG',
        'S/.' => 'PEN',
        'TT$' => 'TTD',
        '₴' => 'UAH',
        'Z$' => 'ZWD',
        'dollar' => 'USD',
        'dollars' => 'USD',

        'stop_words' => ['till', '=', 'i', 'som'],
    ],
    'crypto_currency' => [
        'satoshis' => 'SAT',
        'satoshi' => 'SAT',
        'bitcoins' => 'BTC',
        'ether' => 'ETH',
        'etherium' => 'ETH', //mispelled

        'stop_words' => ['till', '=', 'i', 'som'],
    ],
    'percentage' => [
        'plus' => '+',
        'minus' => '-',
        'som % av' => '%',
        'är hur många % av' => '%',
        'i % av' => '%',

        'stop_words' => ['av', 'i', 'som'],
    ],
    'vat' => [
        'plus' => '+',
        'minus' => '-',
        'skatt' => 'VAT',
        'moms' => 'VAT',

        'stop_words' => ['av','i'],
    ]
];
